searchData={"content_type":"text/markdown","items":[{"doc":"Main user-friendly functions for interacting with the endpoints of a Nominatim API server.\n\nThe `opts` keyword list of the endpoint request functions can optionally include the following:\n* `:base_url` with a string value setting the base URL of the target Nominatim API server (it defaults to the public server).\n* `:force` with a boolean value, where `true` means that validation errors will be ignored and the HTTP GET request to the API will run regardless.\n\n> ##","ref":"ExNominatim.html","title":"ExNominatim","type":"module"},{"doc":">\n> Please respect the [Nominatim Usage Policy](https://operations.osmfoundation.org/policies/nominatim/) when using the public server. To prevent useless requests, it is recommended to not disable request validation.\n\n**New since v1.1.0:** ExNominatim now takes into account your overarching Elixir application's configuration, as defined using the `Config` module. For example, in the `config/config.exs` file of a Phoenix app called `MyApp`, you can define default values like so:\n\n```elixir\nconfig :my_app, MyApp.ExNominatim,\nall: [\n  base_url: \"http://localhost:8080\",\n  force: true,\n  format: \"json\",\n  process: true,\n  atomize: true\n],\nsearch: [format: \"geocodejson\", force: false],\nreverse: [namedetails: 1],\nlookup: [],\ndetails: [],\nstatus: [format: \"json\"]\n```\n\nThe configuration above has the following effects:\n\n* Requests to all endpoints will use the self-hosted Nominatim API instance at port 8080 of `localhost`, accessible over HTTP.\n* Request parameter validation errors (`valid?: false` in the request parameters struct) will be ignored for requests to all endpoints, except for requests to the `/search` endpoint.\n* Unless requested otherwise in `opts`, requests to the `/search` endpoint will return data in GeocodeJSON format (instead of the default `jsonv2`).\n* Requests to the `/reverse` endpoint will set `:namedetails` to 1 (unless otherwise set in `opts`).\n* Requests to the `/status` endpoint will return JSON instead of the [default text](https://nominatim.org/release-docs/develop/api/Status/#output) (HTTP status code 200 and text `OK` or HTTP status 500 and a detailed error mesage).\n* The responses from all endpoints will be processed automatically using `ExNominatim.Report.process/1`, and any maps and contents thereof (or map contents of structs's keys) will be converted from bitstring keys to atom keys using `ExNominatim.Report.atomize/1`.\n\nHere's the preference order:\n\n* The default values for `:base_url` and `:force` across all endpoints are the public Nominatim API server's URL and `false`, respectively.\n* Any keyword-value pairs set in the `:all` list override these defaults and are applied automatically in requests to all endpoints.\n* Any keyword-value pairs set in the list of a specific endpoint override any values defined in the `:all` list and are applied automatically in requests to that endpoint.\n* Anything you set in `opts` overrides all the above.\n\nAt any moment you can use `ExNominatim.get_config/0` to see the current configuration.\n\nThe following things are set as default in ExNominatim's default settings (unless overriden through your own configuration or explicitly-set request parameter values):\n1. The `:format` value for the `/reverse` endpoint, which is switched to GeocodeJSON, instead of XML (the API endpoint's default).\n2. The `:format` value for the `/lookup` endpoint, which is switched to JSONv2, instead of XML (the API endpoint's default).\n3. The `:format` value for the `/status` endpoint, which is switch to JSON, instead of text (the API endpoint's default).","ref":"ExNominatim.html#module-warning-error","title":"Warning {: .error} - ExNominatim","type":"module"},{"doc":"Show internal details for an OpenStreetMap object (for debugging only) using the `/details` API endpoint.\n\nNote: You can set an alternative server as the value of the `:base_url` keyword and ignore any validation errors by setting the `:force` keyword's value to `true`.\n\nGiven a keyword list of request parameters as `opts`, transform this to a `%DetailsParams{}` struct, validate it and its values using the `ExNominatim.Validations` module's functions and, if valid (or if `opts` contains `force: true` to ignore any validation errors), prepare and make an HTTP request to the `/details` endpoint.","ref":"ExNominatim.html#details/1","title":"ExNominatim.details/1","type":"function"},{"doc":"Same as `ExNominatim.explain_fields/1`, but show all fields and their explanations.","ref":"ExNominatim.html#explain_fields/0","title":"ExNominatim.explain_fields/0","type":"function"},{"doc":"Given a request params struct (`%ReverseParams{}`, `%SearchParams{}`, etc.), a keyword list, or a list of atoms corresponding to keys, explain the fields, their default values (if any) and their values' limits (if applicable). It ignores any keyword list keys or atoms in the list that do not correspond to request parameters.\n\nAlternatively, provide the atom corresponding to an endpoint (e.g., `:search` for `/search`) to get the explanations of the parameters specific to that endpoint.\n\nThe content shown is adapted from the [Nominatim API Reference](https://nominatim.org/release-docs/latest/api/Overview/) page of the endpoint corresponding to the request params struct.\n\nThe result is a map, so that you can reuse the explanations for user guidance and error reporting in your own application.","ref":"ExNominatim.html#explain_fields/1","title":"ExNominatim.explain_fields/1","type":"function"},{"doc":"Show the current configuration defaults.","ref":"ExNominatim.html#get_config/0","title":"ExNominatim.get_config/0","type":"function"},{"doc":"Look up address details for OpenStreetMap objects by their \"OSM ID\" using the `/lookup` API endpoint.\n\nNote: You can set an alternative server as the value of the `:base_url` keyword and ignore any validation errors by setting the `:force` keyword's value to `true`.\n\nGiven a keyword list of request parameters as `opts`, transform this to a `%LookupParams{}` struct, validate it and its values using the `ExNominatim.Validations` module's functions and, if valid (or if `opts` contains `force: true` to ignore any validation errors), prepare and make an HTTP request to the `/lookup` endpoint.","ref":"ExNominatim.html#lookup/1","title":"ExNominatim.lookup/1","type":"function"},{"doc":"Search OpenStreetMap objects by their location using the `/reverse` API endpoint.\n\nNote: You can set an alternative server as the value of the `:base_url` keyword and ignore any validation errors by setting the `:force` keyword's value to `true`.\n\nGiven a keyword list of request parameters as `opts`, transform this to a `%ReverseParams{}` struct, validate it and its values using the `ExNominatim.Validations` module's functions and, if valid (or if `opts` contains `force: true` to ignore any validation errors), prepare and make an HTTP request to the `/reverse` endpoint.","ref":"ExNominatim.html#reverse/1","title":"ExNominatim.reverse/1","type":"function"},{"doc":"Search OpenStreetMap objects by name or type using the `/search` API endpoint.\n\nGiven a keyword list of request parameters as `params`, transform this to a `%SearchParams{}` struct, validate it and its values using the `ExNominatim.Validations` module's functions and, if valid (or if `params` contains `force: true` to ignore any validation errors), prepare and make an HTTP request to the `/search` endpoint.","ref":"ExNominatim.html#search/1","title":"ExNominatim.search/1","type":"function"},{"doc":"Query the status of the server at `base_url` using the `/status` API endpoint without any keyword list options provided.","ref":"ExNominatim.html#status/0","title":"ExNominatim.status/0","type":"function"},{"doc":"Query the status of the server at `base_url` using the `/status` API endpoint.\n\nNote: You can set an alternative server as the value of the `:base_url` keyword and ignore any validation errors by setting the `:force` keyword's value to `true`.\n\nGiven a keyword list of request parameters as `opts`, transform this to a `%StatusParams{}` struct, validate it and its values using the `ExNominatim.Validations` module's functions and, if valid (or if `opts` contains `force: true` to ignore any validation errors), prepare and make an HTTP request to the `/status` endpoint.","ref":"ExNominatim.html#status/1","title":"ExNominatim.status/1","type":"function"},{"doc":"Functions for preparing an HTTP request, including validating the base URL of the target Nominatim API server setting the User-Agent header automatically, selecting all the non-nil request parameters, creating a validated request, and dispatching it to the requested endpoint.\n\nThe `opts` keyword list of the endpoint request functions can optionally include the following:\n* `:base_url` with a string value setting the base URL of the target Nominatim API server (it defaults to the public server).\n* `:force` with a boolean value, where `true` means that validation errors will be ignored and the HTTP GET request to the API will run regardless.","ref":"ExNominatim.Client.html","title":"ExNominatim.Client","type":"module"},{"doc":"Use the `/details` API endpoint. Delegated to from `ExNominatim.details/1`, which is documented.","ref":"ExNominatim.Client.html#details/1","title":"ExNominatim.Client.details/1","type":"function"},{"doc":"Use the `/lookup` API endpoint. Delegated to from `ExNominatim.lookup/1`, which is documented.","ref":"ExNominatim.Client.html#lookup/1","title":"ExNominatim.Client.lookup/1","type":"function"},{"doc":"Create a new request params struct from the provided keyword list `opts`, taking into account any required fields listed as atoms in the `required` list. Delegated to from the `new/1` function of the modules of `SearchParams`, `ReverseParams`, etc. (the `module`).","ref":"ExNominatim.Client.html#new/3","title":"ExNominatim.Client.new/3","type":"function"},{"doc":"Prepares an HTTP request to the `endpoint` at `base_url` with the `params` map containing request parameters.\n\n* `endpoint` is one of `:search`, `:reverse`, `:lookup`, `:status`, `:details`.\n* `params` is a map (not a keyword list!) with the request parameters.\n* `base_url` is the the base URL of the target Nominatim API server.\n\nYou can use this function directly if you want to bypass the more user-friendly delegate functions in the main ExNominatim module and define the `params` map directly. This function does not perform any validation of the validity of the keys in `params` for the selected endpoint, or of their respective values vs. the API endpoint's specification.","ref":"ExNominatim.Client.html#prepare/3","title":"ExNominatim.Client.prepare/3","type":"function"},{"doc":"Use the `/reverse` API endpoint. Delegated to from `ExNominatim.reverse/1`, which is documented.","ref":"ExNominatim.Client.html#reverse/1","title":"ExNominatim.Client.reverse/1","type":"function"},{"doc":"Use the `/search` API endpoint. Delegated to from `ExNominatim.search/1`, which is documented.","ref":"ExNominatim.Client.html#search/1","title":"ExNominatim.Client.search/1","type":"function"},{"doc":"Use the `/status` API endpoint without setting any request parameters. Delegated to from `ExNominatim.status/0`, which is documented.","ref":"ExNominatim.Client.html#status/0","title":"ExNominatim.Client.status/0","type":"function"},{"doc":"Use the `/status` API endpoint. Delegated to from `ExNominatim.status/1`, which is documented.","ref":"ExNominatim.Client.html#status/1","title":"ExNominatim.Client.status/1","type":"function"},{"doc":"The struct for a request to the `/details` API endpoint.","ref":"ExNominatim.Client.DetailsParams.html","title":"ExNominatim.Client.DetailsParams","type":"module"},{"doc":"Construct a new `%DetailsParams{}` struct from the content of the keyword list `opts`.","ref":"ExNominatim.Client.DetailsParams.html#new/1","title":"ExNominatim.Client.DetailsParams.new/1","type":"function"},{"doc":"The struct for a request to the `/lookup` API endpoint.","ref":"ExNominatim.Client.LookupParams.html","title":"ExNominatim.Client.LookupParams","type":"module"},{"doc":"Construct a new `%LookupParams{}` struct from the content of the keyword list `opts`, while taking into account the required field (`:osm_ids`) for a request to the `/lookup` API endpoint.","ref":"ExNominatim.Client.LookupParams.html#new/1","title":"ExNominatim.Client.LookupParams.new/1","type":"function"},{"doc":"The struct for a request to the `/reverse` API endpoint.","ref":"ExNominatim.Client.ReverseParams.html","title":"ExNominatim.Client.ReverseParams","type":"module"},{"doc":"Construct a new `%ReverseParams{}` struct from the content of the keyword list `opts`, while taking into account the required fields (`:lat` and `:lon`) for a request to the `/reverse` API endpoint.","ref":"ExNominatim.Client.ReverseParams.html#new/1","title":"ExNominatim.Client.ReverseParams.new/1","type":"function"},{"doc":"The struct for a request to the `/search` API endpoint.","ref":"ExNominatim.Client.SearchParams.html","title":"ExNominatim.Client.SearchParams","type":"module"},{"doc":"Construct a new `%SearchParams{}` struct from the content of the keyword list `opts`, while taking into account that a request to the `/reverse` API endpoint requires either a free-form query with the keyword `:q` assigned a value _or_ a structured query with values defined for at least one of `:amenity`, `:street`, `:city`, `:county`, `:state`, `:country` and `:postalcode`, but not both.","ref":"ExNominatim.Client.SearchParams.html#new/1","title":"ExNominatim.Client.SearchParams.new/1","type":"function"},{"doc":"The struct for a request to the `/status` API endpoint.","ref":"ExNominatim.Client.StatusParams.html","title":"ExNominatim.Client.StatusParams","type":"module"},{"doc":"Construct a new `%StatusParams{}` struct from the content of the keyword list `opts`.","ref":"ExNominatim.Client.StatusParams.html#new/1","title":"ExNominatim.Client.StatusParams.new/1","type":"function"},{"doc":"Functions for reporting processed responses from the Nominatim API.","ref":"ExNominatim.Report.html","title":"ExNominatim.Report","type":"module"},{"doc":"Given a map, a list of maps, or a struct, convert all keys from bitstrings to atoms. Bitstring keys including dashes will have all their dashes replaced with underscores.","ref":"ExNominatim.Report.html#atomize/1","title":"ExNominatim.Report.atomize/1","type":"function"},{"doc":"Convert the output of any of the endpoint functions to a more usable map.","ref":"ExNominatim.Report.html#process/1","title":"ExNominatim.Report.process/1","type":"function"},{"doc":"","ref":"ExNominatim.Report.html#to_atom/1","title":"ExNominatim.Report.to_atom/1","type":"function"},{"doc":"Functions used for the automatic validation of the keys of a request parameters struct according to the target endpoint, their values according to the API endpoint's specification, and invalidate any request parameters with confusing intent that might have unexpected results, such as defining both the `:q` free-form query parameter and at least one of the parameters of a structured query (`:city`, `:country`, etc.).","ref":"ExNominatim.Validations.html","title":"ExNominatim.Validations","type":"module"},{"doc":"Show all fields and their explanations.","ref":"ExNominatim.Validations.html#explain_fields/0","title":"ExNominatim.Validations.explain_fields/0","type":"function"},{"doc":"Given a request params struct (`%ReverseParams{}`, `%SearchParams{}`, etc.), a keyword list, or a list of atoms corresponding to keys, explain the fields, their default values (if any) and their values' limits (if applicable). It ignores any keyword list keys or atoms in the list that do not correspond to request parameters. If provided with the atom of a field, it returns the validation/explanation message for that field","ref":"ExNominatim.Validations.html#explain_fields/1","title":"ExNominatim.Validations.explain_fields/1","type":"function"},{"doc":"Sanitizes the values of those fields of the `params` request params struct (`%SearchParams{}`, `%ReverseParams{}`, etc.) that contain a comma-separated list of strings according to the Nominatim API specification. It collapses all spaces and trims and leading and trailing commas.\n\nYou can use this function directly if you want to combine it with `ExNominatim.Validations.validate/1`.","ref":"ExNominatim.Validations.html#sanitize_comma_separated_strings/1","title":"ExNominatim.Validations.sanitize_comma_separated_strings/1","type":"function"},{"doc":"Validates the content and intent of a request represented by a request parameters struct `params` (`%SearchParams{}`, `%ReverseParams{}`, etc.).\n\nYou can use this function directly if you want to combine it with `ExNominatim.HTTP.prepare/3`.","ref":"ExNominatim.Validations.html#validate/1","title":"ExNominatim.Validations.validate/1","type":"function"},{"doc":"<!--   -->\n\n# ExNominatim\n\n**ExNominatim** is a full-featured client for the [OpenStreetMap](https://www.openstreetmap.org) [Nominatim API V1](https://nominatim.org/release-docs/latest/api/Overview/), with extensive request validation, robust error-handling and reporting, and user guidance with helpful validation messages.","ref":"readme.html","title":"ExNominatim","type":"extras"},{"doc":"- Prevent unnecessary calls to the Nominatim API server by validating intended requests and preventing them if the request parameters are invalid.\n- Solid error-handling for robustness in production.\n- Provide helpful validation messages to the user when a request is deemed invalid.","ref":"readme.html#goals","title":"Goals - ExNominatim","type":"extras"},{"doc":"- Covers the `/search`, `/reverse`, `/lookup`, `/status` and `/details` endpoints.\n- Utilizes request parameter structs with the appropriate fields (except for `json_callback`) for each endpoint.\n- Configurable for your application with overridable defaults using Elixir's `Config` module to set any default values, including the `:base_url` option for use with self-hosted Nominatim API instances.\n- Validates parameter values prior to the request (possible to override this with the `force: true` option).\n- Provides helpful return tuples `{:ok, ...}`, `{:error, reason}` and `{:error, {specific_error, other_info}}` across the board.\n- Collects all detected field validation errors in an `:errors` field, and provides a `:valid?` field in each request params struct.\n- Automatically sets the `User-Agent` header to \"ExNominatim/{version}\" to comply with the [Nominatim Usage Policy](https://operations.osmfoundation.org/policies/nominatim/).","ref":"readme.html#features","title":"Features - ExNominatim","type":"extras"},{"doc":"The package can be installed [from Hex](https://hex.pm/package/ex_nominatim) by adding `ex_nominatim` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ex_nominatim, \"~> 1.1.2\"}\n  ]\nend\n```\n\nThe code can be found on [Github/waseigo/ex_nominatim](https://github.com/waseigo/ex_nominatim).\n\nDocumentation has been published on [HexDocs](https://hexdocs.pm/ex_nominatim).\n\nThere is also a thread open on the Elixir Programming Language Forum: [ExNominatim - A full-featured client for the OpenStreetMap Nominatim API V1](https://elixirforum.com/t/exnominatim-a-full-featured-client-for-the-openstreetmap-nominatim-api-v1/65120/1).","ref":"readme.html#installation","title":"Installation - ExNominatim","type":"extras"},{"doc":"By calling the endpoint functions of the `ExNominatim` module you will be hitting the public Nominatim API server with each endpoint's default options as described in the API documentation; i.e., all requests use   as the value of `:base_url` in `opts` and the default parameters for each endpoint are handled by the API according to its documentation.\n\n**Please respect the [Nominatim Usage Policy](https://operations.osmfoundation.org/policies/nominatim/) when using the public server.**","ref":"readme.html#usage","title":"Usage - ExNominatim","type":"extras"},{"doc":"In the more likely scenario where you use ExNominatim in your own application (e.g., in a Phoenix application) called `MyApp`, you can override all defaults across all endpoints and then even for each endpoint through your application's configuration, e.g. in the `config/config.exs` file of a Phoenix app. For example:\n\n```elixir\nconfig :my_app, MyApp.ExNominatim,\n  all: [\n    base_url: \"http://localhost:8080\",\n    force: true,\n    format: \"json\",\n    process: true,\n    atomize: true\n  ],\n  search: [format: \"geocodejson\", force: false],\n  reverse: [namedetails: 1],\n  lookup: [],\n  details: [],\n  status: [format: \"json\"]\n```\n\nThe configuration above has the following effects:\n\n- Requests to all endpoints will use the self-hosted Nominatim API instance at port 8080 of `localhost`, accessible over HTTP.\n- Request parameter validation errors (`valid?: false` in the request parameters struct) will be ignored for requests to all endpoints, except for requests to the `/search` endpoint.\n- Unless requested otherwise in `opts`, requests to the `/search` endpoint will return data in GeocodeJSON format (instead of the default `jsonv2`).\n- Requests to the `/reverse` endpoint will set `:namedetails` to 1 (unless otherwise set in `opts`).\n- Requests to the `/status` endpoint will return JSON instead of the [default text](https://nominatim.org/release-docs/develop/api/Status/#output) (HTTP status code 200 and text `OK` or HTTP status 500 and a detailed error mesage).\n- The responses from all endpoints will be processed automatically using `ExNominatim.Report.process/1`, and any maps and contents thereof (or map contents of structs's keys) will be converted from bitstring keys to atom keys using `ExNominatim.Report.atomize/1`.\n\nRefer to [the documentation of the main `ExNominatim` module](https://hexdocs.pm/ex_nominatim/ExNominatim.html) for more information.","ref":"readme.html#optional-configuration-and-default-parameters","title":"Optional configuration and default parameters - ExNominatim","type":"extras"},{"doc":"- Build [Cachex](https://hexdocs.pm/cachex/Cachex.html) in (see [Reactive Warming](https://hexdocs.pm/cachex/reactive-warming.html)) and provide the option to automatically throttle requests to the public API to the \"absolute maximum of 1 request per second\" as requested by the [Nominatim Usage Policy](https://operations.osmfoundation.org/policies/nominatim/).\n- Implement a test suite.","ref":"readme.html#ideas-and-someday-maybe-features","title":"Ideas and someday/maybe features - ExNominatim","type":"extras"},{"doc":"Copyright 2024, made by [Isaak Tsalicoglou](https://linkedin.com/in/tisaak), [OVERBRING](https://overbring.com) in [Athens](https://www.openstreetmap.org/#map=11/37.9909/23.7387), [Attica](https://www.openstreetmap.org/#map=8/37.061/23.456), [Greece](https://www.openstreetmap.org/#map=6/38.310/24.489).\n\nMany thanks to all the volunteers and contributors of [OpenStreetMap](https://www.openstreetmap.org/) and [Nominatim](https://nominatim.org/).","ref":"readme.html#who-made-this","title":"Who made this? - ExNominatim","type":"extras"}],"producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}